function [x y, out] = cont_framework_tv_w(opts,solver)
%CONT_FRAMEOWRK Continuation framework for pdNCG_TV_W.

tic; % start counting time.

%Initialize total number of pdNCG iterations.
total_itr = 0;

% Initialize data.
order_mu     = max(-floor(log(abs(opts.mu))./log(10)),0);
order_tau_W  = max(-floor(log(abs(opts.tau_W))./log(10)),0);
order_tau_TV = max(-floor(log(abs(opts.tau_TV))./log(10)),0);
    
t_mu     = max(order_mu-1,0);
t_tau_W  = max(order_tau_W-1,0);
t_tau_TV = max(order_tau_TV-1,0);

if t_mu == 0
    mus_vector = ones(opts.cont_steps,1).*opts.mu;
else
    mus_vector = logspace(log10(0.1),log10(opts.mu),opts.cont_steps);
end

if t_tau_W == 0
    taus_W_vector = ones(opts.cont_steps,1).*opts.tau_W;
else
    taus_W_vector = logspace(log10(0.1),log10(opts.tau_W),opts.cont_steps);
end

if t_tau_TV == 0
    taus_TV_vector = ones(opts.cont_steps,1).*opts.tau_TV;
else
    taus_TV_vector = logspace(log10(0.1),log10(opts.tau_TV),opts.cont_steps);
end

% Begin continuation.
for i=1:opts.cont_steps
 
    opts.mu  = mus_vector(i);  
    opts.tau_W = taus_W_vector(i);
    opts.tau_TV = taus_TV_vector(i);
    
    if opts.verbose >= 1
        fprintf('%5s:  %d \n','Total continuation steps',opts.cont_steps);
        fprintf('%5s: %d \n','Current continuation step',i);   
        fprintf('%5s: %d \n','Current mu    ',opts.mu);
        fprintf('%5s: %d \n','Current tau_TV',opts.tau_TV); 
        fprintf('%5s: %d \n','Current tau_W ',opts.tau_W);
    end
    
    [x y_W, y_TV, out] = solver(opts);
    opts.x0    = x;  
    opts.y_W0  = y_W;
    opts.y_TV0 = y_TV;
    
    total_itr = total_itr + out.itr;
end

out.itr = total_itr;

% Put all dual variables in one vector.
y = [y_W;y_TV];

% Stop measuring time and save total time.
out.time = toc;