function opts = checkOptions(opts)
%CHECKOPTION Checks the input arguments for pdNCG.

if nargin < 1
    error('At least one argument is required.');
end

if ~isfield(opts, {'tolpcg'})  
    opts.tolpcg = 1.0e-1;  
else
    if opts.tolpcg <= 0 || opts.tolpcg >= 1
        error('The tolerance of PCG, i.e. opts.tolpcg, must be greater than 0 and less than 1.'); 
    end
end

if ~isfield(opts, {'mxitrpcg'})  
    opts.mxitrpcg = 10000;
else
    if opts.mxitrpcg <= 0
        error('The maximum iterations of PCG, i.e. opts.mxitrpcg, must be positive.'); 
    end
end

if ~isfield(opts, {'reg'})  
    opts.reg = 1.0e-2;
else
    if opts.reg <= 0
        error('The regularization parameter, i.e. opts.reg, of the second derivatives must be greater than 0.'); 
    end
end

if ~isfield(opts, {'maxstepsize'})  
    opts.maxstepsize = 1.0;
else
    if opts.maxstepsize <= 0 || opts.maxstepsize > 1
        error('The maximum stepsize, i.e. opts.maxstepsize, must be greater than 0 and less or equal to 1.'); 
    end
end

if ~isfield(opts, {'c2'})  
    opts.c2 = 1.0e-3;
else
    if opts.c2 <= 0 || opts.c2 >= 0.5
        error('The constant c2 of the backtracking algorithm must be positive and less than 0.5.'); 
    end
end

if ~isfield(opts, {'c3'})  
    opts.c3 = 0.9;
else
    if opts.c3 <= 0 || opts.c3 >=1
        error('The constant c3 of the backtracking algorithm must be positive and less than one.'); 
    end
end

if ~isfield(opts, {'max_itr_bt'})  
    opts.max_itr_bt = 10;
else
    if opts.max_itr_bt <= 0
        error('The maximum iterations of the backtracking algorithm must be positive.'); 
    end
end

if ~isfield(opts, {'tol'})         
    opts.tol = 1.0e-2; 
else
    if opts.tol < 0
        error('The termination tolerance opts.tol must be non-negative.'); 
    end    
end

if ~isfield(opts, {'mu'})  
    text0 = '****************************************************************************\n';
    text1 = 'WARNING!!! The approximation parameter opts.mu is not provided.\n';
    text2 = 'opts.mu is set to the default value 1.0e-5. This means that \n';
    text3 = 'the method cannot deliver a solution with rellative error less than 1.0e-5. \n';
    text4 = '****************************************************************************\n \n';
    fprintf(text0);
    fprintf(text1);
    fprintf(text2);
    fprintf(text3);
    fprintf(text4);
    opts.mu = 1.0e-5;
else
    if opts.mu <= 0, 
        error('The approximation parameter mu, i.e. opts.mu, must be greater than 0.'); 
    end
end

if opts.mu >= opts.tol,
    
    opts.mu = opts.tol*1.0e-1;
        
    text0 = '********************************************************************************\n';
    text1 = 'WARNING!!! The approximation parameter opts.mu is less than or equal to opt.tol.\n';
    text2 = 'The method cannot deliver the required accuracy with this setting.\n';
    text3 = 'Parameter opts.mu is overwritten with opts.tol*1.0e-1.\n';
    text4 = '********************************************************************************\n \n';
    fprintf(text0);
    fprintf(text1);
    fprintf(text2);
    fprintf(text3);
    fprintf(text4);
end

if ~isfield(opts, {'maxpdNCGitr'})
    opts.maxpdNCGitr = 50;     
else
    if opts.tol <= 0
        error('The maximum iterations of pdNCG opts.maxpdNCGitr must be positive.'); 
    end      
end

if ~isfield(opts, {'verbose'}), opts.verbose = 2; end

if strcmp(opts.problem,'ls') || strcmp(opts.problem,'lg') || ...
   strcmp(opts.problem,'wavelet') || strcmp(opts.problem,'tv') || ...
   strcmp(opts.problem,'user')
   opts = checkOptions_ls_lg_user_wavelet_tv(opts);
end
if strcmp(opts.problem,'tv_wavelet')
    opts = checkOptions_TV_W(opts);
end