#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <ctime>

#include "mpi_gen.h"
#include "mpi_matvec.h"

void Generator::strongly_convex(int p, int size_p, long long n, long long m, long long k,
                                long long n_p, long long m_p, long long k_p, double *xopt_p, double *s_val_p,
                                double theta, double eig_lo, double eig_up,
                                double *b_noisy_p, double G[][2])
{
    double *sub_g_p = new double[n_p];
    double *output = new double[n_p];
    double *e_p = new double[m_p];
    double *b_p = new double[m_p];
    
    double abs_val;
    double local_max,   local_min;
    double global_max, global_min;
    
    Matvec matvec;
    
    MPI_Status status;
    
    /* initialize random seed: */
    srand (time(NULL) + p*pow(3,10));
//    srand (p*pow(3,10)); // for testing.
    
    /* Create part of singular values for processor p */
    for (long long i = 0; i < n_p; i = i + 2)
        s_val_p[i] = eig_lo;
    
    for (long long i = 1; i < n_p; i = i + 2)
        s_val_p[i] = eig_up;
    
    /* Initialize few stuff */
    for (long long i = 0; i < n_p; i++)
        xopt_p[i]  = 0;
    
//    for (long long i = 0; i < 2; i++)
//    {
//        for (long long j = 0; j < 2; j++)
//        {
//            printf("G_%d[%lld][%lld] = %e \n", p, i, j, G[i][j]);
//        }
//    }
    
    for (long long j = 0; j < k_p; j = j + 2)
    {
        if (j + k_p*p <= k)
            xopt_p[j] = -100*eig_up;
    }
    
    for (long long j = 1; j < k_p; j = j + 2)
    {
        if (j + k_p*p <= k)
            xopt_p[j] = 1*eig_lo;
    }
    
//    for (long long i = 0; i < n_p; i++)
//        printf("xopt_%d[%lld] = %e \n", p, i, xopt_p[i]);
    
    /* Create part of subgradient of l1-norm at xopt for processor p */
    for (long long i = 0; i < n_p; i++)
    {
        if (xopt_p[i] > 0)
            sub_g_p[i] =  1.0;
        else if (xopt_p[i] < 0)
            sub_g_p[i] = -1.0;
        else
        {
            sub_g_p[i] = ((double)rand()/(double)RAND_MAX);
            sub_g_p[i] = sub_g_p[i]*2 - 1;
        }
    }
    
//    for (long long i = 0; i < n_p; i++)
//        printf("sub_g_%d[%lld] = %e \n", p, i, sub_g_p[i]);
    
    /* Calculate noise vector for processor p */
    matvec.matvec_G_transpose(n_p, G, sub_g_p, output);
    
    for (long long i = 0; i < n_p; i++)
    {
        e_p[i] = output[i]/s_val_p[i];
//        printf("e_%d[%lld] = %e \n", p, i, e_p[i]);
    }
    
    for (long long i = n_p; i < m_p; i++)
    {
        e_p[i] = ((double)rand()/(double)RAND_MAX);
        e_p[i] = e_p[i]*2 - 1;
//        printf("e_%d[%lld] = %e \n", p, i, e_p[i]);
    }
    
    /* Calculate noiseless b for processor p */
    matvec.matvec_A_forward(n_p, m_p, G, s_val_p, xopt_p, b_p);

    /* Calculate noisy b for processor p */
    for (long long i = 0; i < m_p; i++)
    {
        b_noisy_p[i] = b_p[i] + e_p[i];
//        printf("b_noisy_%d[%lld] = %e \n", p, i, b_noisy_p[i]);
    }
    
//    double delete_me[n_p];
//    matvec.matvec_A_transpose(n_p, m_p, G, s_val_p, b_noisy_p, delete_me);
//    
//    for (long int i = 0; i < n_p; i++)
//        printf("Atb_n_%d[%lld] = %e \n", p, i, delete_me[i]);
    
    delete[] sub_g_p;
    delete[] output;
    //delete[] temp;
    //delete[] temp2;
    delete[] e_p;
    delete[] b_p;
    
}