#include <stdio.h>

#include "mpi_matvec.h"

void Matvec::matvec_G_forward(long long n_p, double G[][2], double *input, double *output)
{
    for (long long q = 0; q < n_p/2; q++)
    {
        long long index = 2*q;
        
        for (long long i = 0 ; i < 2; i++)
        {
            output[index + i] = 0;
            
            for (long long j = 0; j < 2; j++)
                output[index + i] += G[i][j]*input[index + j];
        }
    }
}

void Matvec::matvec_G_transpose(long long n_p, double G[][2], double *input, double *output)
{
    for (long long q = 0; q < n_p/2; q++)
    {
        long long index = 2*q;
        
        for (long long i = 0 ; i < 2; i++)
        {
            output[index + i] = 0;
            
            for (long long j = 0; j < 2; j++)
                output[index + i] += G[j][i]*input[index + j];
        }
    }
}

void Matvec::matvec_A_forward(long long n_p, long long m_p, double G[][2], double *s_val_p, double *input, double *output)
{
    matvec_G_transpose(n_p, G, input, output);
    
    for (long long i = 0; i < n_p; i++)
        output[i] = output[i]*s_val_p[i];
    
    for (long long i = n_p; i < m_p; i++)
        output[i] = 0;
}

void Matvec::matvec_A_transpose(long long n_p, long long m_p, double G[][2], double *s_val_p, double *input, double *output)
{
    double *temp = new double[n_p];
    
    for (long long i = 0; i < n_p; i++)
        temp[i] = input[i]*s_val_p[i];
    
    matvec_G_forward(n_p, G, temp, output);
    
    delete[] temp;
}